from __future__ import annotations

from crashtest.contracts.has_solutions_for_exception import HasSolutionsForException
from crashtest.contracts.provides_solution import ProvidesSolution
from crashtest.contracts.solution import Solution
from crashtest.contracts.solution_provider_repository import (
    SolutionProviderRepository as BaseSolutionProviderRepository,
)


class SolutionProviderRepository(BaseSolutionProviderRepository):
    def __init__(self, solution_providers: list[type] | None = None) -> None:
        self._solution_providers: list[type] = []

        if solution_providers is None:
            solution_providers = []

        self.register_solution_providers(solution_providers)

    def register_solution_provider(
        self, solution_provider_class: type
    ) -> SolutionProviderRepository:
        self._solution_providers.append(solution_provider_class)

        return self

    def register_solution_providers(
        self, solution_provider_classes: list[type]
    ) -> SolutionProviderRepository:
        for solution_provider_class in solution_provider_classes:
            self.register_solution_provider(solution_provider_class)

        return self

    def get_solutions_for_exception(self, exception: Exception) -> list[Solution]:
        solutions: list[Solution] = []

        if isinstance(exception, Solution):
            solutions.append(exception)

        if isinstance(exception, ProvidesSolution):
            solutions.append(exception.solution)

        for solution_provider_class in self._solution_providers:
            if not issubclass(solution_provider_class, HasSolutionsForException):
                continue

            solution_provider: HasSolutionsForException = solution_provider_class()

            try:
                if not solution_provider.can_solve(exception):
                    continue
            except Exception:
                continue

            try:
                for solution in solution_provider.get_solutions(exception):
                    solutions.append(solution)
            except Exception:
                continue

        return solutions



from __future__ import annotations

from typing import TYPE_CHECKING
from typing import Any
from typing import List


if TYPE_CHECKING:
    from crashtest.frame import Frame


class FrameCollection(List[Any]):
    def __init__(self, frames: list[Frame] | None = None, count: int = 0) -> None:
        if frames is None:
            frames = []

        super().__init__(frames)

        self._count = count

    @property
    def repetitions(self) -> int:
        return self._count - 1

    def is_repeated(self) -> bool:
        return self._count > 1

    def increment_count(self, increment: int = 1) -> FrameCollection:
        self._count += increment

        return self

    def compact(self) -> list[FrameCollection]:
        """
        Compacts the frames to deduplicate recursive calls.
        """
        collections = []
        current_collection = FrameCollection()

        i = 0
        while i < len(self) - 1:
            frame = self[i]
            if frame in self[i + 1 :]:
                duplicate_indices = []
                for sub_index, sub_frame in enumerate(self[i + 1 :]):
                    if frame == sub_frame:
                        duplicate_indices.append(sub_index + i + 1)

                found_duplicate = False
                for duplicate_index in duplicate_indices:
                    collection = FrameCollection(self[i:duplicate_index])
                    if collection == current_collection:
                        current_collection.increment_count()
                        i = duplicate_index
                        found_duplicate = True
                        break

                if found_duplicate:
                    continue

                collections.append(current_collection)
                current_collection = FrameCollection(self[i : duplicate_indices[0]])

                i = duplicate_indices[0]

                continue

            if current_collection.is_repeated():
                collections.append(current_collection)
                current_collection = FrameCollection()

            current_collection.append(frame)
            i += 1

        collections.append(current_collection)

        return collections
